<?xml version="1.0"?>

<st-source>
<time-stamp>From VisualWorks® NonCommercial, 7.6 of lundi 3 mars 2008 on mardi 19 mars 2013 at 7:02:56</time-stamp>


<methods>
<class-id>CormasNS.Kernel.Cormas class</class-id> <category>+ utilities - mathematics</category>

<body package="cormas" selector="lowerQuartile:">lowerQuartile: aCollectionOfNumbers 
"Purpose : Returns the lower quartile of a numbers distribution. 
The lower quartile (or first quartile or Q1) is the value of the sorted series of numbers which splits the lowest 25% of data
For example, assume that we have obtained the following 20 observations:
   2, 4, 7, -20, 22, -1, 0, -1, 7, 15, 8, 4, -4, 11, 11, 12, 3, 12, 18, 1
In order to calculate the quartiles we first have to sort the observations:
   -20, -4, -1, -1, 0, 1, 2, 3, 4, 4, 7, 7, 8, 11, 11, 12, 12, 15, 18, 22
The position of the first quartile is x = round(0.25*(20+1)) = round(5.25) = 5, which means that Q1 is the 5th value of the sorted series, namely Q1 = 0.
Example: Cormas lowerQuartile: #(2 4 7 -20 22 -1 0 -1 7 15 8 4 -4 11 11 12 3 12 18 1)"
	| sortedColl |
	aCollectionOfNumbers size = 0 ifTrue: [^0].
	sortedColl := aCollectionOfNumbers asSortedCollection: [:a :b | a &lt; b].
	^sortedColl at: ((0.25 * (sortedColl size + 1)) rounded)</body>
</methods>

<methods>
<class-id>CormasNS.Kernel.Cormas class</class-id> <category>+ utilities - mathematics</category>

<body package="cormas" selector="median:">median: aCollectionOfNumbers 
	"Answer the median of a numbers distribution. 
	The median is defined by :
		the element whose index is (N+1)/2, if N is odd	or
		the average of elements whose indexes are N/2 and 1+N/2, if N is even
aCollectionOfNumbers = &lt; OrderedCollection&gt; or &lt;Array&gt;
ex:		Cormas median: #( 1 2 3) 	=&gt; 2
		Cormas median: #( 1 2 3 5) 	=&gt; 2.5"

	| sortedColl |
	aCollectionOfNumbers size = 0 ifTrue: [^0].
	sortedColl := aCollectionOfNumbers asSortedCollection: [:a :b | a &gt; b].
	sortedColl size odd 
		ifTrue: [^sortedColl at: (sortedColl size + 1) / 2]
		ifFalse: 
			[^((sortedColl at: sortedColl size / 2) 
				+ (sortedColl at: sortedColl size / 2 + 1)) / 2]</body>
</methods>

<methods>
<class-id>CormasNS.Kernel.Cormas class</class-id> <category>+ utilities - mathematics</category>

<body package="cormas" selector="upperQuartile:">upperQuartile: aCollectionOfNumbers 
"Purpose : Returns the upper quartile of a numbers distribution. 
The upper quartile (or third quartile or Q3) is the value of the sorted series of numbers which splits the highest 25% of data (or lowest 75%)
For example, assume that we have obtained the following 20 observations:
   2, 4, 7, -20, 22, -1, 0, -1, 7, 15, 8, 4, -4, 11, 11, 12, 3, 12, 18, 1
In order to calculate the quartiles we first have to sort the observations:
   -20, -4, -1, -1, 0, 1, 2, 3, 4, 4, 7, 7, 8, 11, 11, 12, 12, 15, 18, 22
The position of the third quartile is x = round(0.25*(20+1)) = round(15.75) = 16, which means that Q3 is the 16th value of the sorted series, namely Q3 = 12.
Example: Cormas upperQuartile: #(2 4 7 -20 22 -1 0 -1 7 15 8 4 -4 11 11 12 3 12 18 1)"
	| sortedColl |
	aCollectionOfNumbers size = 0 ifTrue: [^0].
	aCollectionOfNumbers size = 1 ifTrue: [^aCollectionOfNumbers first].
	sortedColl := aCollectionOfNumbers asSortedCollection: [:a :b | a &lt; b].
	^sortedColl at: ((0.75 * (sortedColl size + 1)) rounded)</body>
</methods>

</st-source>
